# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# coding=utf-8

"""
Acs ERROR CODE module.
"""

SDK_INVALID_REGION_ID = 'SDK.InvalidRegionId'
SDK_SERVER_UNREACHABLE = 'SDK.ServerUnreachable'
SDK_INVALID_REQUEST = 'SDK.InvalidRequest'
SDK_MISSING_ENDPOINTS_FILER = 'SDK.MissingEndpointsFiler'
SDK_UNKNOWN_SERVER_ERROR = 'SDK.UnknownServerError'
SDK_INVALID_CREDENTIAL = 'SDK.InvalidCredential'
SDK_INVALID_SESSION_EXPIRATION = 'SDK.InvalidSessionExpiration'
SDK_GET_SESSION_CREDENTIAL_FAILED = 'SDK.GetSessionCredentialFailed'
SDK_INVALID_PARAMS = 'SDK.InvalidParams'
SDK_NOT_SUPPORT = 'SDK.NotSupport'
SDK_ENDPOINT_RESOLVING_ERROR = 'SDK.EndpointResolvingError'
SDK_ENDPOINT_TESTABILITY = 'SDK.EndpointTestability'
SDK_HTTP_ERROR = 'SDK.HttpError'
SDK_INVALID_PARAMETER = "SDK.InvalidParameter"
